import base64
from fastapi import APIRouter, Depends, HTTPException
from typing import List
import schemas
from users import User, UserDB, UserTable, current_user
from db import get_db
import models
from sqlalchemy.orm import Session as DbSession, lazyload
from sqlalchemy.orm.exc import NoResultFound
from sqlalchemy.exc import IntegrityError
from fastapi import status
from sqlalchemy.sql import select

router = APIRouter()


@router.post("/sessions",
             response_model=schemas.Session,
             tags=["sessions"],
             status_code=status.HTTP_201_CREATED)
def create_session(session: schemas.SessionBase,
                   db: DbSession = Depends(get_db),
                   user: User = Depends(current_user)):
    session_props = session.dict()
    session_props['user'] = user.id
    session_props['data'] = base64.b64decode(session_props['data'])
    db_obj = models.Session(**session_props)
    db.add(db_obj)
    db.commit()
    return db_obj


@router.get("/sessions", response_model=List[schemas.Session], tags=["sessions"])
def list_sessions(skip=0,
                  limit=100,
                  db: DbSession = Depends(get_db),
                  user: User = Depends(current_user)):
    return db.query(models.Session).filter(models.Session.user == user.id).order_by(
        models.Session.start_time.desc()).offset(skip).limit(limit).all()


@router.post("/friends/request_friendship/{user_id}", tags=["friends"])
def create_friend_request(other_user_id: str,
                          db: DbSession = Depends(get_db),
                          user: User = Depends(current_user)):
    if models.Friendship.are_friends(db, other_user_id, user.id):
        raise HTTPException(status.HTTP_406_NOT_ACCEPTABLE, detail="already friends")

    FR = models.FriendRequest
    friend_request_from_other_user = db.query(FR).filter(FR.requesting_user == other_user_id,
                                                         FR.receiving_user == user.id).count()
    if friend_request_from_other_user > 0:
        raise HTTPException(status.HTTP_406_NOT_ACCEPTABLE,
                            detail="Friend request exist from other user, accept it")
    else:
        try:
            new_friend_request = FR(requesting_user=user.id, receiving_user=other_user_id)
            db.add(new_friend_request)
            db.commit()
            return {"msg": "ok"}
        except IntegrityError:
            raise HTTPException(status.HTTP_406_NOT_ACCEPTABLE,
                                detail="Friend request already exists")


@router.post("/friends/accept_friendship/{user_id}", tags=["friends"])
def accept_friend_request(other_user_id: str,
                          db: DbSession = Depends(get_db),
                          user: User = Depends(current_user)):
    FR = models.FriendRequest
    try:
        friend_request = db.query(FR).filter(FR.requesting_user == other_user_id,
                                             FR.receiving_user == user.id).one()
    except NoResultFound:
        raise HTTPException(status_code=404, detail="No matching friend request found")

    models.Friendship.befriend(db, other_user_id, user.id)
    db.delete(friend_request)
    db.commit()
    return {"msg": "ok"}


@router.get("/friends", tags=["friends"], response_model=schemas.FriendsInfo)
def list_friends_info(db: DbSession = Depends(get_db), user: User = Depends(current_user)):
    user_obj = db.query(UserTable).filter(UserTable.id == user.id).one()
    return schemas.FriendsInfo(incoming_requests=user_obj.friend_requests_in,
                               outgoing_requests=user_obj.friend_requests_out)


# todo: remove friend requests
# todo: remove friendship
# todo: search user by email
# todo: add usernames to users
# todo: search by username