from fastapi_users import FastAPIUsers, models
from fastapi_users.db import SQLAlchemyBaseUserTable, SQLAlchemyUserDatabase
from fastapi_users.authentication import JWTAuthentication
from config import JWT_SECRET
from fastapi import Request
from db import database, Base
from sqlalchemy.orm import relationship, backref
from sqlalchemy import Integer, Column
from fastapi_users.models import BaseUser


class User(models.BaseUser):
    pass


class UserCreate(models.BaseUserCreate):
    pass


class UserUpdate(User, models.BaseUserUpdate):
    pass


class UserDB(User, models.BaseUserDB):
    pass


class UserTable(Base, SQLAlchemyBaseUserTable):
    #id = Column(Integer, primary_key=True)
    sessions = relationship("Session")
    friend_requests_in = relationship(
        "UserTable",
        secondary="friend_request",
        primaryjoin=("UserTable.id == FriendRequest.receiving_user"),
        secondaryjoin=("UserTable.id == FriendRequest.requesting_user"),
        backref=backref("friend_requests_out"))
    friends = relationship('UserTable',
                           secondary="friendship",
                           primaryjoin=("UserTable.id == Friendship.user_id"),
                           secondaryjoin=("UserTable.id == Friendship.friend_id"))


user_db = SQLAlchemyUserDatabase(UserDB, database, UserTable.__table__)
jwt_authentication = JWTAuthentication(secret=JWT_SECRET,
                                       lifetime_seconds=60 * 60 * 8,
                                       tokenUrl="auth/jwt/login")

fastapi_users = FastAPIUsers(
    user_db,
    [jwt_authentication],
    User,
    UserCreate,
    UserUpdate,
    UserDB,
)
current_user = fastapi_users.current_user(active=True, verified=True)
current_superuser = fastapi_users.current_user(active=True, superuser=True, verified=True)


def on_after_register(user: UserDB, request: Request):
    print(f"User {user.id} has registered.")


def on_after_forgot_password(user: UserDB, token: str, request: Request):
    print(f"User {user.id} has forgot their password. Reset token: {token}")


def after_verification_request(user: UserDB, token: str, request: Request):
    print(f"Verification requested for user {user.id}. Verification token: {token}")


def add_user_routers(app):
    app.include_router(fastapi_users.get_auth_router(jwt_authentication),
                       prefix="/auth/jwt",
                       tags=["auth"])

    app.include_router(fastapi_users.get_register_router(on_after_register),
                       prefix="/auth",
                       tags=["auth"])
    app.include_router(
        fastapi_users.get_reset_password_router(JWT_SECRET,
                                                after_forgot_password=on_after_forgot_password),
        prefix="/auth",
        tags=["auth"],
    )
    app.include_router(
        fastapi_users.get_verify_router(JWT_SECRET,
                                        after_verification_request=after_verification_request),
        prefix="/auth",
        tags=["auth"],
    )
    app.include_router(fastapi_users.get_users_router(), prefix="/users", tags=["users"])
